/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
  Stan Borinski
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <varargs.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"
#include "attributes.h"
#include "objects.h"
#include "llist.h"
#include "rwhois_supp.h"
#include "parse.h"
#include "client_msgs.h"
#include "validate.h"

/***********
  global prototypes
************/

attribute_struct *link_object_to_attribute();

/************
  local prototypes
*************/
static int format_check();
static char *get_number();
static char *get_macro_name();
static char *get_macro_format();


/*************************************************************************
 validate_record - checks against:
   1) all same object
   2) required attributes filled in
   3) data in correct format
**************************************************************************/
int validate_record (ifp, ofp, record_attribs)
FILE *ifp, *ofp;
llist_struct *record_attribs;

{
int retval;
char object_hold[MAX_LINE], object[MAX_LINE], attribute[MAX_LINE];
char datum[MAX_LINE];
int junk;
attribute_struct *attrib;
line_struct *line_info;
int local_error = FALSE, found = FALSE, hold_row_no;
llist_struct *top = record_attribs;

/********
  first make sure same object, does have attribute, and has correct format
********/

*object_hold = '\0';
while (record_attribs != NULL) {
  line_info = record_attribs->datum;
  retval = get_tuple(object, attribute, datum, line_info->line);
  if (!strlen (object_hold))
    strcpy (object_hold, object);
  if (strcasecmp (object, object_hold) != 0) {
    local_error = TRUE;
    print_error (INVALID_ATTRIBUTE, "%d", line_info->row_no);
    }
  else if ((attrib = link_object_to_attribute (object, attribute))== NULL) {
    local_error = TRUE;
    print_error (INVALID_ATTRIBUTE, "%d", line_info->row_no);
    }
/**
  else if (!format_check (attrib->format, datum))
    local_error = TRUE;
    print_error (INVALID_FORMAT, "%d", line_info->row_no);
    print_error_to_file (ifp, ofp, FALSE, line_info->row_no,
      "Invalid format for '%s'", attribute);
    }
***/
  record_attribs = record_attribs->nxt;
  }

/********
  make sure have all the required attributes and no multi-lines that
  are not defined as such
********/

if (!local_error) {
  attrib = get_object_attributes_by_object_name (object);
  while (attrib != NULL) {
    if (attrib->required) {
      record_attribs = top;
      found = FALSE;
      while (record_attribs != NULL && !found) {
        line_info = record_attribs->datum;
        if (get_tuple(object, attribute, datum, line_info->line)) {
          if (strcasecmp (attribute, attrib->name) == 0 ||
            (attrib->name_alias != NULL &&
            strcasecmp (attribute, attrib->name_alias) ==0))
            found = TRUE;
          }
        record_attribs = record_attribs->nxt;
        }
      if (!found) {
        local_error = TRUE;
        print_error (MISSING_REQ_ATTRIB, "%d", line_info->row_no);
        }
      }
    if (!attrib->mult_lines) {
      record_attribs = top;
      found = FALSE;
      while (record_attribs != NULL) {
        line_info = record_attribs->datum;
        if (get_tuple(object, attribute, datum, line_info->line)) {
          if (strcasecmp (attribute, attrib->name) == 0 ||
            (attrib->name_alias != NULL && 
            strcasecmp (attribute, attrib->name_alias) ==0)) {
            if (!found++) {
              hold_row_no = line_info->row_no;
              }
            }
          }
        record_attribs = record_attribs->nxt;
        }
      if (found > 1) {
        local_error = TRUE;
        print_error (MISSING_REL_OBJECT, "%d", line_info->row_no);
/**
        print_error_to_file (ifp, ofp, FALSE, hold_row_no,
          "Attribute '%s' appears multiple times", attrib->name);
**/
        }
      }
    attrib = attrib->nxt;
    }
  }
if (local_error)
  return (FALSE);
return (TRUE);
}

/*************************************************************************
  returns TRUE if attribute is a part of the object
          FALSE otherwise
**************************************************************************/
attribute_struct *link_object_to_attribute (object, attribute)
char *object;
char *attribute;

{
attribute_struct *attrib = NULL;

if ((attrib = get_object_attributes_by_object_name (object))!= NULL)
  return (find_specific_attribute (attrib, attribute));
return (NULL);
}

/*************************************************************************
  dumps the error file with message at the bottom
**************************************************************************/
print_error_to_file (va_alist)
va_dcl
{
va_list list;
FILE *ifp, *ofp;
int fatal;
int row_no;
char *format;
char line[MAX_LINE];

va_start(list);
ifp = va_arg(list, FILE*);
ofp = va_arg(list, FILE*);
fatal = va_arg(list, int);
row_no = va_arg(list, int);
format = va_arg(list, char*);
fprintf (ofp, "** ERROR (line %d) ", row_no);
vfprintf(ofp, format, list);
fprintf (ofp, "\n");
va_end(list);
if (fatal) {
  while (fgets (line, MAX_LINE, ifp) != NULL) 
    fputs (line, ofp);
  }
}

/* ------------------------------------------------
   Routines from here down developed by Stan Borinski
   Modified by Mark Kosters
   -------------------------------------------------- */

/*************************************************************************
 format_check - checks the data against the format specifications
**************************************************************************/
static int format_check(format_string, data)
char *format_string;
char *data;
{
char *ptr;
int number;
char macro[MAX_LINE], macro_format[MAX_LINE];

ptr = format_string;
if (*ptr != '%') {
  printf("format string <%s> doesn't start with '%%'\n", format_string);
  return(FALSE);
  } 
else
  ptr++;

while (*ptr) {
  switch(*ptr) {
      case '%':  /* start of new format */
        break;
      case '-':  /* alignment */
        break;
      case '0': case '1': case '2': case '3': case '4': /* length */
      case '5': case '6': case '7': case '8': case '9':
        ptr = get_number(ptr, &number);
        printf("number = %d, ptr = <%c>\n", number, *ptr);
        break;
      case 'B':  /* bit size */
        break;
      case 'c':  /* character */
        break;
      case 's':  /* string */
        break;
      case 'd': case 'i':  /* decimal integer */
        break;
      case 'M':  /* macro */
        /* ptr = get_macro_name(ptr, macro); */
        sscanf(++ptr, "%s", macro);
        printf("macro <%s>, ptr = <%c>\n", macro, *ptr);
        get_macro_format(macro, macro_format);
        break;
      case 'x': case 'o': case 'f': case 'e':  /* Unused for now */
        break;
      default:  /* Assume a literal */
        break;
      /***************  Don't forget enumerated types ************** */
    }
    ptr++;
  }

} /* END check_format */


/***************************************************************************
 get_number - get the next number from the format string; reset ptr
**************************************************************************/
static char *
get_number(ptr, number)
char *ptr; 
int *number;
{
  /* Courtesy of K&R, page 97 */
for (*number = 0; isdigit(*ptr); ptr++)
  *number = 10 * *number + (*ptr - '0');

return(ptr);
}


/***************************************************************************
 get_macro_name - get macro name
**************************************************************************/
static char *
get_macro_name(ptr, macro)
char *ptr; 
char *macro;
{
  ptr++; /* Step past 'M' */

  while ((*macro++ = *ptr++) != '=')
    ;

  *(--macro) = '\0';  /* Nuke '=' and tie it off */
  ptr++;        /* Step past '=' */
  if (*ptr != '[')
    ptr--;      /* Hmm, no bracket */

  return(ptr);
}


/***************************************************************************
 get_format_string - get the format string of the object attribute
**************************************************************************/
static char *
get_format_string(object, attribute, format_string)
char *object, *attribute, *format_string;
{
  /*******  The macros may be several layers deep ****** */
  /* format_string = get_macro_format(macro, format_string); */
  return (format_string);
}

/***************************************************************************
 get_macro_format - get the format string of the current macro
**************************************************************************/
static char *
get_macro_format(macro, format)
char *macro, *format;

{
  strcpy(format, "[%32s]");
}

